#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <pgpKeys.h>
#include <pgpRandomPool.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "implementation.h"
#include "global.h"
#include "action.h"
#include "varie.h"

void
on_new_file1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  close_file();
  PGPNewKeySet(pgpContext, &pgpKeySet);
}


void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

  GtkWidget *selector = create_selector("Open file...", GTK_SIGNAL_FUNC(open_filename));
  gtk_widget_show(selector);
}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

  GtkWidget *selector = create_selector("Save file...", GTK_SIGNAL_FUNC(save_as_filename));
  gtk_widget_show(selector);
}


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *about_dlg = create_about();
  gnome_dialog_run_and_close(GNOME_DIALOG(about_dlg));
}


gboolean
on_app1_delete_event                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_main_quit();
  return FALSE;
}


void
on_ctree1_drag_data_received           (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
  printf("I'm being gragged!\n");
}


void
on_encode1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *encode_dlg = create_encode();
  gint result;

  result = gnome_dialog_run(GNOME_DIALOG(encode_dlg));

  switch (result) {
    case 0: {
      PGPKeyID *signKeyID = NULL;
      PGPKeySetRef recKeySet = kInvalidPGPKeySetRef;
      PGPKeySetRef tmpKeySet = kInvalidPGPKeySetRef;
      PGPKeyRef key;
      PGPError err;
 
      GtkWidget *sign = lookup_widget(encode_dlg, "sign");
      GtkWidget *encrypt = lookup_widget(encode_dlg, "encrypt");
      GtkWidget *detached = lookup_widget(encode_dlg, "detached");
      GtkWidget *clearsign = lookup_widget(encode_dlg, "clear");
      GtkWidget *albero = lookup_widget(app1, "ctree1");

      GtkWidget *file_entry = lookup_widget(encode_dlg, "file");

      GList *l;

      gchar *filename;
      gboolean detach, clear;

      NodeData *data;

      filename = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(file_entry), TRUE);

      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sign))) {
        signKeyID = &defSignKeyID;
        detach = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(detached));
        clear = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(clearsign));
      }

      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(encrypt))) {

        PGPNewKeySet(pgpContext, &recKeySet);
        l = GTK_CTREE(albero)->clist.selection;
        if (l) {
          do {
            data = (NodeData *)(GTK_CTREE_ROW(l->data)->row.data);
            if (data && (data->tipo == KEYID)) {

              err = PGPGetKeyByKeyID(pgpKeySet, &data->keyID, kPGPPublicKeyAlgorithm_Invalid, &key);
              if (IsntPGPError(err)) {
                PGPNewSingletonKeySet(key, &tmpKeySet);
                PGPAddKeys(tmpKeySet, recKeySet);
                PGPFreeKeySet(tmpKeySet);
              }
            }
            l = g_list_next(l);
          } while (l);
        }
      }
      gnome_dialog_close(GNOME_DIALOG(encode_dlg));

      if (filename) {
        encode(filename, signKeyID, detach, clear, recKeySet);
      }

      g_free(filename);
      if (PGPKeySetRefIsValid(recKeySet)) PGPFreeKeySet(recKeySet);

      break;
    }
    case 1: {
      gnome_dialog_close(GNOME_DIALOG(encode_dlg));
      break;
    }
    default: {
      break;
    }
  }

}

gboolean
on_entry1_key_press_event              (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  GtkWidget *d, *l;

  PGPGlobalRandomPoolAddKeystroke(event->keyval);

  d = (GtkWidget *)user_data;
  l = lookup_widget(d, "label6");
  
  view_entropy(l);

  return TRUE;
}

void
on_entropy1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *entropy_dlg = create_entropy();
  GtkWidget *l;

  l = lookup_widget(entropy_dlg, "label6");
  
  view_entropy(l);

  gnome_dialog_run_and_close(GNOME_DIALOG(entropy_dlg));

}

gboolean
on_entropy_motion_notify_event         (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
  GtkWidget *d, *l;

  PGPGlobalRandomPoolMouseMoved();

  d = (GtkWidget *)user_data;
  l = lookup_widget(d, "label6");
  
  view_entropy(l);

  return FALSE;
}

void
on_image1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *image_dlg;
  GtkWidget *pm;
  GtkWidget *albero = lookup_widget(app1, "ctree1");

  GList *l;

  NodeData *data;
  int fd, size, start;
  char filename[32];

  l = GTK_CTREE(albero)->clist.selection;
  if (l) {
    do {
      data = (NodeData *)(GTK_CTREE_ROW(l->data)->row.data);
      if (data && (data->tipo == PHOTO)) {
      
        sprintf(filename, "%s", "/home/andrea/asipgpXXXXXX");
        fd = mkstemp(filename);

        if (fd != -1) {
          size = data->photo.size;
          start = data->photo.data[0];
          write(fd, &data->photo.data[start], size - start);
          close(fd);

          image_dlg = create_foto();
          pm = lookup_widget(image_dlg, "pixmap1");
          gnome_pixmap_load_file(GNOME_PIXMAP(pm), filename);
          remove(filename);
          gnome_dialog_run_and_close(GNOME_DIALOG(image_dlg));
        }
      }
      l = g_list_next(l);
    } while (l);
  }
}

void
on_open_key_pair1_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint result;
  GtkWidget *key_pair_dlg = create_key_pair();

  result = gnome_dialog_run(GNOME_DIALOG(key_pair_dlg));
  
  switch (result) {
    case 0: {
      gchar *pub, *sec;
      GtkWidget *pub_entry = lookup_widget(key_pair_dlg, "public");
      GtkWidget *sec_entry = lookup_widget(key_pair_dlg, "secret");
      
      pub = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(pub_entry), TRUE);
      sec = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(sec_entry), TRUE);
      gnome_dialog_close(GNOME_DIALOG(key_pair_dlg));

      if (pub && sec) {
        parse_key_pair(pub, sec);
      }

      g_free(pub);
      g_free(sec);

      break;
    }
    case 1: {
      gnome_dialog_close(GNOME_DIALOG(key_pair_dlg));
      break;
    }
    default: {
      break;
    }
  }
}


void
on_decode1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *selector = create_selector("Open file...", GTK_SIGNAL_FUNC(open_filename_decode));
  gtk_widget_show(selector);
}

void
on_hash1_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint result;
  PGPHashAlgorithm algID;
  GtkWidget *hash_dlg = create_hash();

  result = gnome_dialog_run(GNOME_DIALOG(hash_dlg));
  
  switch (result) {
    case 0: {
      GtkWidget *file_entry = lookup_widget(hash_dlg, "file");
      GtkWidget *algo;
      gchar *filename;
      
      filename = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(file_entry), TRUE);
      algo = lookup_widget(hash_dlg, "md5");
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(algo))) algID = kPGPHashAlgorithm_MD5;
      algo = lookup_widget(hash_dlg, "sha");
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(algo))) algID = kPGPHashAlgorithm_SHA;
      algo = lookup_widget(hash_dlg, "ripemd160");
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(algo))) algID = kPGPHashAlgorithm_RIPEMD160;
      gnome_dialog_close(GNOME_DIALOG(hash_dlg));

      if (filename) {
        hash_file(filename, algID);
      }

      g_free(filename);

      break;
    }
    case 1: {
      gnome_dialog_close(GNOME_DIALOG(hash_dlg));
      break;
    }
    default: {
      break;
    }
  }
}

void
on_default1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  parse_default();
}


void
on_signing_key1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *albero = lookup_widget(app1, "ctree1");
  GList *l;
  NodeData *data;

  l = GTK_CTREE(albero)->clist.selection;
  data = (NodeData *)(GTK_CTREE_ROW(l->data)->row.data);
  if (data && (data->tipo == KEYID)) {
    defSignKeyID = data->keyID;
  }
}


void
on_sign_toggled                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  GtkWidget *dialog = (GtkWidget *)user_data;
  GtkWidget *sign = lookup_widget(dialog, "sign");
  GtkWidget *options = lookup_widget(dialog, "options");
  GtkWidget *encrypt = lookup_widget(dialog, "encrypt");
  GtkWidget *detached = lookup_widget(dialog, "detached");

  gtk_widget_set_sensitive(options, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sign)));
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(encrypt))) {
    GtkWidget *none = lookup_widget(dialog, "none");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(detached))) {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(none), TRUE);
    }
  }
  gtk_widget_set_sensitive(detached, !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(encrypt)));
}


void
on_encrypt_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  on_sign_toggled(togglebutton, user_data);
}

void
on_unselect_all1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *albero = lookup_widget(app1, "ctree1");
  gtk_clist_unselect_all(GTK_CLIST(albero));
}

void
on_symmetric1_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint result;
  PGPCipherAlgorithm algID;
  GtkWidget *symmetric_dlg = create_symmetric();

  result = gnome_dialog_run(GNOME_DIALOG(symmetric_dlg));
  
  switch (result) {
    case 0: {
      GtkWidget *file_entry = lookup_widget(symmetric_dlg, "file");
      GtkWidget *algo, *key_entry;
      gchar *filename, *key;
      gboolean encrypt;
      
      filename = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(file_entry), TRUE);

      algo = lookup_widget(symmetric_dlg, "cast5");
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(algo))) algID = kPGPCipherAlgorithm_CAST5;

      algo = lookup_widget(symmetric_dlg, "des");
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(algo))) algID = kPGPCipherAlgorithm_3DES;

      algo = lookup_widget(symmetric_dlg, "idea");
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(algo))) algID = kPGPCipherAlgorithm_IDEA;

      algo = lookup_widget(symmetric_dlg, "encrypt");
      encrypt = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(algo));

      key_entry = lookup_widget(symmetric_dlg, "key");
      key = gtk_editable_get_chars(GTK_EDITABLE(key_entry), 0, -1);

      gnome_dialog_close(GNOME_DIALOG(symmetric_dlg));

      if (filename) {
        encrypt_decrypt_file(filename, algID, key, encrypt);
      }

      g_free(filename);

      break;
    }
    case 1: {
      gnome_dialog_close(GNOME_DIALOG(symmetric_dlg));
      break;
    }
    default: {
      break;
    }
  }
}
