#include <gnome.h>

#include <pgpRandomPool.h>
#include <pgpErrors.h>
#include <pgpUtilities.h>

#include "support.h"
#include "interface.h"
#include "parsing.h"
#include "varie.h"
#include "global.h"
#include "eventi.h"

static PGPError entropyEvent(PGPContextRef context, PGPEvent *event) {
  PGPError pgpResult = kPGPError_NoErr;
  GtkWidget *entropy_dlg = create_entropy();
  GtkWidget *l;

  l = lookup_widget(entropy_dlg, "label6");
  view_entropy(l);

  if (gnome_dialog_run_and_close(GNOME_DIALOG(entropy_dlg)) != 0) {
    pgpResult = 1000;
  }
  return pgpResult;
}

static PGPError passphraseEvent(PGPContextRef context, PGPEvent *event) {
  GtkWidget *pass_dlg = create_passphrase();
  GtkWidget *lista = lookup_widget(pass_dlg, "clist1");
  gint result;
      
  if (!event->data.passphraseData.fConventional) {
    populate_lista(event->data.passphraseData.keyset, lista);
  }
      
  result = gnome_dialog_run(GNOME_DIALOG(pass_dlg));
  switch (result) {
    case 0: {
      GtkWidget *pass_entry = lookup_widget(pass_dlg, "pass");
      gchar *pp = gtk_editable_get_chars(GTK_EDITABLE(pass_entry), 0, -1);
      PGPAddJobOptions(event->job, 
        PGPOPassphrase(context, pp),
        PGPOLastOption(context));
      g_free(pp);

      gnome_dialog_close(GNOME_DIALOG(pass_dlg));
      break;
    } 
    case 1: {
      gnome_dialog_close(GNOME_DIALOG(pass_dlg));
      break;
    }
    default: {
      break;
    }
  }
  return kPGPError_NoErr;
}

static PGPError outputEvent(PGPContextRef context, PGPEvent *event) {
  PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;
  PGPError err;

  GtkWidget *save_dlg, *tipo, *nome, *file_entry;
  static char *tipi[] = {"Encrypted", "Signed", "DetachedSignature", "Key", "Unknown", "X509"};
  gint result;
  gchar *buf;
  char temp[10];
      
  save_dlg = create_save_output();
  tipo = lookup_widget(save_dlg, "tipo");
  file_entry = lookup_widget(save_dlg, "file");
      
  sprintf(temp, "File type: %d 0x%x", event->data.outputData.messageType, event->data.outputData.messageType);
  gtk_label_set_text(GTK_LABEL(tipo), temp);
  nome = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(file_entry));
  if (event->data.outputData.suggestedName) {
    gtk_entry_set_text(GTK_ENTRY(nome), event->data.outputData.suggestedName);
  }
      
  result = gnome_dialog_run(GNOME_DIALOG(save_dlg));
  switch (result) {
    case 0: {
      buf = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(file_entry), FALSE);
      err = PGPNewFileSpecFromFullPath(context, buf, &outFileRef);
      if (IsntPGPError(err)) {
        PGPAddJobOptions(event->job, 
          PGPOOutputFile(context, outFileRef),
          PGPOLastOption(context));
      }
      g_free(buf);

      gnome_dialog_close(GNOME_DIALOG(save_dlg));
      break;
    }
    case 1: {
      PGPAddJobOptions(event->job, 
        PGPODiscardOutput(context, TRUE),
        PGPOLastOption(context));
           
      gnome_dialog_close(GNOME_DIALOG(save_dlg));
      break;
    }
    default: {
      PGPAddJobOptions(event->job, 
        PGPODiscardOutput(context, TRUE),
        PGPOLastOption(context));
      break;
    }
  }
  if (PGPFileSpecRefIsValid(outFileRef)) PGPFreeFileSpec(outFileRef);

  return kPGPError_NoErr;
}  

static PGPError keyFoundEvent(PGPContextRef context, PGPEvent *event) {
  PGPError err;
  PGPUInt32 numKeys;
  GtkWidget *albero;
  GtkCTreeNode *file_n;

  albero = lookup_widget(app1, "ctree1");
  clear_buf();
  buf_par[0] = "Imported keys";
 
  err = PGPCountKeys(event->data.keyFoundData.keySet, &numKeys);
  if (IsPGPError(err)) {
    sprintf(temp_par[2], "%d", err);
  } else {
    sprintf(temp_par[2], "%d", numKeys);
  }

  file_n = gtk_ctree_insert_node(GTK_CTREE(albero), NULL, NULL, buf_par,
    0, NULL, NULL, NULL, NULL, FALSE, TRUE);

  parse_key_set(event->data.keyFoundData.keySet, file_n, albero);

  PGPAddKeys(event->data.keyFoundData.keySet, pgpKeySet);
  PGPCommitKeyRingChanges(pgpKeySet);

  return kPGPError_NoErr;
}

static PGPError detachedSignatureEvent(PGPContextRef context, PGPEvent *event) {
  PGPFileSpecRef sigFileRef = kInvalidPGPFileSpecRef;
  PGPError err;

  GtkWidget *sign_dlg, *nome;
  gint result;
  gchar *buf;
      
  sign_dlg = create_detached();
      
  result = gnome_dialog_run(GNOME_DIALOG(sign_dlg));
  switch (result) {
    case 0: {
      nome = lookup_widget(sign_dlg, "signature");
      buf = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(nome), TRUE);
      err = PGPNewFileSpecFromFullPath(context, buf, &sigFileRef);
      if (IsntPGPError(err)) {
        PGPAddJobOptions(event->job, 
          PGPODetachedSig(context,
            PGPOInputFile(context, sigFileRef),
            PGPOLastOption(context)),
          PGPOLastOption(context));
      }
      g_free(buf);

      gnome_dialog_close(GNOME_DIALOG(sign_dlg));
      break;
    }
    case 1: {
      gnome_dialog_close(GNOME_DIALOG(sign_dlg));
      break;
    }
    default: {
      break;
    }
  }
  if (PGPFileSpecRefIsValid(sigFileRef)) PGPFreeFileSpec(sigFileRef);

  return kPGPError_NoErr;
}  

static PGPError signatureEvent(PGPContextRef context, PGPEvent *event) {
  PGPError err;
  PGPSize used;

  GtkWidget *signature_dlg = create_signature();
  GtkWidget *id = lookup_widget(signature_dlg, "id");
  GtkWidget *name = lookup_widget(signature_dlg, "name");
  GtkWidget *status = lookup_widget(signature_dlg, "status");
  GtkWidget *date = lookup_widget(signature_dlg, "date");

  char outString[kPGPMaxKeyIDStringSize];
  char buf[128], *tmp;

  PGPGetKeyIDString(&event->data.signatureData.signingKeyID, kPGPKeyIDString_Full, outString);
  gtk_label_set_text(GTK_LABEL(id), outString);

  err = PGPGetPrimaryUserIDNameBuffer(event->data.signatureData.signingKey, sizeof(buf), buf, &used);
  if (IsntPGPError(err)) {
    gtk_label_set_text(GTK_LABEL(name), buf);
  }
  
  if (event->data.signatureData.verified) {
    tmp = "Signature OK!";
  } else {
    tmp = "Invalid Signature!!";
  }

  gtk_label_set_text(GTK_LABEL(status), tmp);

  create_time(kPGPError_NoErr, event->data.signatureData.creationTime, buf);
  gtk_label_set_text(GTK_LABEL(date), buf);

  gnome_dialog_run_and_close(GNOME_DIALOG(signature_dlg));

  return kPGPError_NoErr;
}

static PGPError recipientsEvent(PGPContextRef context, PGPEvent *event) {
  GtkWidget *missing_dlg, *lista;
  char outString[kPGPMaxKeyIDStringSize];
  gint i;
      
  if (event->data.recipientsData.keyCount) {
    missing_dlg = create_missing();
    lista = lookup_widget(missing_dlg, "lista");
  
    for (i = 0; i < event->data.recipientsData.keyCount; i++) {
      PGPGetKeyIDString(&event->data.recipientsData.keyIDArray[i], kPGPKeyIDString_Full, outString);
      buf_par[0] = outString;
      gtk_clist_append(GTK_CLIST(lista), buf_par);
    }

    gnome_dialog_run_and_close(GNOME_DIALOG(missing_dlg));
  }
      
  return kPGPError_NoErr;
}

PGPError myEventHandler(PGPContextRef context, PGPEvent *event, PGPUserValue userValue) {
  PGPError pgpResult = kPGPError_NoErr;
 
  switch (event->type) {
  	
    case kPGPEvent_InitialEvent: {
      GtkWidget *appBar = lookup_widget(app1, "appbar1");
      gnome_appbar_set_progress(GNOME_APPBAR(appBar), 0);
      break;
    }
    
    case kPGPEvent_NullEvent: {
      GtkWidget *appBar = lookup_widget(app1, "appbar1");
      gfloat completionPercent;

      if (event->data.nullData.bytesTotal != 0) {
        completionPercent = (1.0 * event->data.nullData.bytesWritten) / event->data.nullData.bytesTotal;
      } else {
        completionPercent = 1;
      }

      gnome_appbar_set_progress(GNOME_APPBAR(appBar), completionPercent);
      gtk_main_iteration_do(FALSE);
      break;
    }
    
    case kPGPEvent_FinalEvent: {
      GtkWidget *appBar = lookup_widget(app1, "appbar1");
      gnome_appbar_set_progress(GNOME_APPBAR(appBar), 0);
      break;
    }
    
    case kPGPEvent_EntropyEvent: {
      pgpResult = entropyEvent(context, event);
      break;
    }
    
    case kPGPEvent_ErrorEvent: {
      show_error(event->data.errorData.error);
      break;
    }
    
    case kPGPEvent_WarningEvent: {
      show_error(event->data.warningData.warning);
      break;
    }
    
    case kPGPEvent_PassphraseEvent: {
      pgpResult = passphraseEvent(context, event);
      break;
    }
    case kPGPEvent_KeyFoundEvent: {
      pgpResult = keyFoundEvent(context, event);
      break;
    }
    case kPGPEvent_OutputEvent: {
      pgpResult = outputEvent(context, event);
      break;
    }
    case kPGPEvent_DetachedSignatureEvent: {
      pgpResult = detachedSignatureEvent(context, event);
      break;
    }
    case kPGPEvent_SignatureEvent: {
      pgpResult = signatureEvent(context, event);
      break;
    }
    case kPGPEvent_RecipientsEvent: {
      pgpResult = recipientsEvent(context, event);
      break;
    }
    default: {
      printf("Evento sconosciuto: %d\n", event->type);
      break;
    }
    
  }  
  return pgpResult;
}

void show_error(PGPError err) {
  GtkWidget *appbar = lookup_widget(app1, "appbar1");
  char buf[128];
  
  PGPGetErrorString(err, sizeof(buf), buf);
  gnome_appbar_set_status(GNOME_APPBAR(appbar), buf);
}















