#include <pgpUtilities.h>
#include <pgpKeys.h>
#include <pgpEncode.h>
#include <pgpRandomPool.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "global.h"
#include "support.h"
#include "implementation.h"
#include "parsing.h"
#include "eventi.h"
#include "varie.h"

void init_pgp() {
  PGPsdkInit();
  PGPNewContext(kPGPsdkAPIVersion, &pgpContext);
  PGPNewKeySet(pgpContext, &pgpKeySet);
  memset(&defSignKeyID, 0, sizeof(defSignKeyID));
  PGPNewMemoryMgr(0, &pgpMemoryMgr);
}

void done_pgp() {
  if (PGPKeySetRefIsValid(pgpKeySet)) PGPFreeKeySet(pgpKeySet);
  if (PGPMemoryMgrRefIsValid(pgpMemoryMgr)) PGPFreeMemoryMgr(pgpMemoryMgr);
  if (PGPContextRefIsValid(pgpContext)) PGPFreeContext(pgpContext);
  PGPsdkCleanup();
}

void init_drag_and_drop() {
  GtkTargetEntry target_table[] = {
    {"text/plain", 0, 0},
    {"text/uri-list", 0, 0}
  };
  GtkWidget *albero = lookup_widget(app1, "ctree1");
  gtk_drag_dest_set(albero, GTK_DEST_DEFAULT_ALL, target_table, 2, GDK_ACTION_COPY);
}

void close_file() {
  GtkWidget *albero = lookup_widget(app1, "ctree1");
  gtk_clist_clear(GTK_CLIST(albero));
  if (PGPKeySetRefIsValid(pgpKeySet)) {
    PGPFreeKeySet(pgpKeySet);
  }
  memset(&defSignKeyID, 0, sizeof(defSignKeyID));
}

static void parse_file(const char *filename) {
  PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;
  PGPError err;
  PGPUInt32 numKeys;

  GtkWidget *albero;
  GtkCTreeNode *file_n;

  err = PGPNewFileSpecFromFullPath(pgpContext, filename, &inFileRef);
  if (IsntPGPError(err)) {
    close_file();
    err = PGPOpenKeyRing(pgpContext, kPGPKeyRingOpenFlags_Mutable,
      inFileRef, &pgpKeySet);

    /*    err = PGPImportKeySet(pgpContext, &pgpKeySet, 
      PGPOInputFile(pgpContext, inFileRef), 
      PGPOSendNullEvents(pgpContext, 100),
      PGPOEventHandler(pgpContext, myEventHandler, NULL),
      PGPOLastOption(pgpContext)); */
      
    if (IsntPGPError(err)) {
      albero = lookup_widget(app1, "ctree1");
      
      clear_buf();
      buf_par[0] = filename;

      PGPCountKeys(pgpKeySet, &numKeys);
      if (IsPGPError(err)) {
        sprintf(temp_par[2], "%d", err);
      } else {
        sprintf(temp_par[2], "%d", numKeys);
      }

      file_n = gtk_ctree_insert_node(GTK_CTREE(albero), NULL, NULL, buf_par,
        0, NULL, NULL, NULL, NULL, FALSE, TRUE);

      parse_key_set(pgpKeySet, file_n, albero);
    }
  }
  if (PGPFileSpecRefIsValid(inFileRef)) PGPFreeFileSpec(inFileRef);
  show_error(err);
}

static void save_key_set(const char *filename) {
  PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;
  PGPError err;

  err = PGPNewFileSpecFromFullPath(pgpContext, filename, &outFileRef);
  if (IsntPGPError(err)) {
    err = PGPExportKeySet(pgpKeySet, 
      PGPOOutputFile(pgpContext, outFileRef), 
      PGPOExportPrivateKeys(pgpContext, TRUE),
      PGPOSendNullEvents(pgpContext, 100),
      PGPOEventHandler(pgpContext, myEventHandler, NULL),
      PGPOExportFormat(pgpContext, kPGPExportFormat_Basic),
      PGPOLastOption(pgpContext));
  }
  if (PGPFileSpecRefIsValid(outFileRef)) PGPFreeFileSpec(outFileRef);
  show_error(err);
}

void open_filename(GtkWidget *sender, gpointer user_data) {
  gchar *filename;
  GtkWidget *selector = (GtkWidget *)user_data;

  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(selector));
  gtk_widget_hide(GTK_WIDGET(selector));
  parse_file(filename);
  gtk_widget_destroy(GTK_WIDGET(selector));
}

void save_as_filename(GtkWidget *sender, gpointer user_data) {
  gchar *filename;
  GtkWidget *selector = (GtkWidget *)user_data;

  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(selector));
  gtk_widget_hide(GTK_WIDGET(selector));
  save_key_set(filename);
  gtk_widget_destroy(GTK_WIDGET(selector));
}

GtkWidget *create_selector(const char *msg, GtkSignalFunc func) {
  GtkWidget *open;
  
  open = gtk_file_selection_new(msg);
  gtk_window_set_modal(GTK_WINDOW(open), TRUE);
    
  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(open)->cancel_button),
    "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), open);
  
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(open)->ok_button),
    "clicked", func, open);
  
  return open;
}

void parse_key_pair(const char *public, const char *secret) {
  PGPFileSpecRef pubFileRef = kInvalidPGPFileSpecRef;
  PGPFileSpecRef secFileRef = kInvalidPGPFileSpecRef;

  PGPError err;
  PGPUInt32 numKeys;

  GtkWidget *albero;
  GtkCTreeNode *file_n;

  err = PGPNewFileSpecFromFullPath(pgpContext, public, &pubFileRef);
  if (IsntPGPError(err)) {
    err = PGPNewFileSpecFromFullPath(pgpContext, secret, &secFileRef);
    if (IsntPGPError(err)) {
      close_file();
      err = PGPOpenKeyRingPair(pgpContext, kPGPKeyRingOpenFlags_Mutable,
        pubFileRef, secFileRef, &pgpKeySet);
      if (IsntPGPError(err)) {
        albero = lookup_widget(app1, "ctree1");
        clear_buf();
        buf_par[0] = "key pair";
 
        PGPCountKeys(pgpKeySet, &numKeys);
        if (IsPGPError(err)) {
          sprintf(temp_par[2], "%d", err);
        } else {
          sprintf(temp_par[2], "%d", numKeys);
        }

        file_n = gtk_ctree_insert_node(GTK_CTREE(albero), NULL, NULL, buf_par,
          0, NULL, NULL, NULL, NULL, FALSE, TRUE);

        parse_key_set(pgpKeySet, file_n, albero);
      }
    }
  }
  if (PGPFileSpecRefIsValid(pubFileRef)) PGPFreeFileSpec(pubFileRef);
  if (PGPFileSpecRefIsValid(secFileRef)) PGPFreeFileSpec(secFileRef);
  show_error(err);
}

void parse_default() {
  PGPError err;
  PGPUInt32 numKeys;

  GtkWidget *albero;
  GtkCTreeNode *file_n;
  
  close_file();
  err = PGPOpenDefaultKeyRings(pgpContext, kPGPKeyRingOpenFlags_Create ||  kPGPKeyRingOpenFlags_Mutable, &pgpKeySet);
      
  if (IsntPGPError(err)) {
    albero = lookup_widget(app1, "ctree1");
    PGPCountKeys(pgpKeySet, &numKeys);
      
    clear_buf();
    buf_par[0] = "default key ring";

    if (IsPGPError(err)) {
      sprintf(temp_par[2], "%d", err);
    } else {
      sprintf(temp_par[2], "%d", numKeys);
    }

    file_n = gtk_ctree_insert_node(GTK_CTREE(albero), NULL, NULL, buf_par,
      0, NULL, NULL, NULL, NULL, FALSE, TRUE);

    parse_key_set(pgpKeySet, file_n, albero);
  }
  show_error(err);
}
