#include <pgpUtilities.h>
#include <pgpKeys.h>
#include <pgpEncode.h>

#include "global.h"
#include "support.h"
#include "parsing.h"
#include "varie.h"

static void parse_sig(PGPKeySetRef keySet, PGPSigRef sig, GtkCTreeNode *nodo, GtkWidget *albero) {
  PGPError err;
  PGPSize used;
  PGPKeyRef key;
  PGPTime data;
  PGPKeyID keyID;
  char outString[kPGPMaxKeyIDStringSize];

  clear_buf();
  
  err = PGPGetSigCertifierKey(sig, keySet, &key);
  if (IsPGPError(err)) {
    sprintf(temp_par[0], "%d", err);
    err = PGPGetKeyIDOfCertifier(sig, &keyID);
  } else {
    err = PGPGetPrimaryUserIDNameBuffer(key, sizeof(temp_par[0]), temp_par[0], &used);
    if (IsPGPError(err)) {
      sprintf(temp_par[0], "%d", err);
    } else {
      temp_par[0][used] = '\0';
    }
    err = PGPGetKeyIDFromKey(key, &keyID);
  }

  if (IsPGPError(err)) {
    sprintf(temp_par[1], "%d", err);
  } else {
    err = PGPGetKeyIDString(&keyID, kPGPKeyIDString_Full, outString);
    if (IsPGPError(err)) {
      sprintf(temp_par[1], "%d", err);
    } else {
      buf_par[1] = outString;
    }
  }

  err = PGPGetSigTime(sig, kPGPSigPropCreation, &data);
  create_time(err, data, temp_par[3]);
  
  err = PGPGetSigTime(sig, kPGPSigPropExpiration, &data);
  create_time(err, data, temp_par[4]);
    
  gtk_ctree_insert_node(GTK_CTREE(albero), nodo, NULL, buf_par, 0, 
     NULL, NULL, NULL, NULL, TRUE, TRUE);
}

static void parse_userid(PGPKeySetRef keySet, PGPKeyIterRef keyIter, PGPUserIDRef userID, GtkCTreeNode *nodo, GtkWidget *albero) {
  PGPError err;
  PGPSize used;
  GtkCTreeNode *uid_n;
  PGPSigRef sig;
  PGPBoolean vf;
  PGPInt32 num;
  NodeData *nd = NULL;
  int start;
  
  char *tipi[] = {"Image", "IPAddress", "DNSName", "Notation", "Unknown"}; 
  
  clear_buf();

  err = PGPGetUserIDBoolean(userID, kPGPUserIDPropIsAttribute, &vf);
  if (vf) {
    err = PGPGetUserIDNumber(userID, kPGPUserIDPropAttributeType, &num);
    if (IsPGPError(err)) {
      sprintf(temp_par[0], "%d", err);
    } else {
      switch (num) {
        case kPGPAttribute_Image: {
          buf_par[0] = tipi[0];
          nd = malloc(PHOTO_SIZE(4096));
          nd->tipo = PHOTO;
          err = PGPGetUserIDStringBuffer(userID, 
            kPGPUserIDPropAttributeData, 4096, &nd->photo.data[0], &nd->photo.size);
          if (IsPGPError(err)) {
            sprintf(temp_par[1], "%d", err);
            free(nd);
            nd = NULL;
          } else {
            start = nd->photo.data[0];
            sprintf(temp_par[1], "%d - %d = %d", nd->photo.size, start, nd->photo.size - start);
          }
          break;
        }
        case kPGPAttribute_IPAddress: {
          buf_par[0] = tipi[1];
          break;
        }
        case kPGPAttribute_DNSName: {
          buf_par[0] = tipi[2];
          break;
        }
        case kPGPAttribute_Notation: {
          buf_par[0] = tipi[3];
          break;
        }
        default: {
          buf_par[0] = tipi[4];
          break;
        }
      }
    }
    
  } else {
    err = PGPGetUserIDStringBuffer(userID, 
      kPGPUserIDPropName, sizeof(temp_par[0]), temp_par[0], &used);
    if (IsPGPError(err)) {
      sprintf(temp_par[0], "%d", err);
    } else {
      temp_par[0][used] = '\0';
    }
    err = PGPGetUserIDStringBuffer(userID, 
      kPGPUserIDPropCommonName, sizeof(temp_par[1]), temp_par[1], &used);
    if (IsPGPError(err)) {
      sprintf(temp_par[1], "%d", err);
    } else {
      temp_par[1][used] = '\0';
    }
    err = PGPGetUserIDStringBuffer(userID, 
      kPGPUserIDPropEmailAddress, sizeof(temp_par[2]), temp_par[2], &used);
    if (IsPGPError(err)) {
      sprintf(temp_par[2], "%d", err);
    } else {
      temp_par[2][used] = '\0';
    }
  }

  uid_n = gtk_ctree_insert_node(GTK_CTREE(albero), nodo, NULL, buf_par, 0, 
    NULL, NULL, NULL, NULL, FALSE, FALSE);
  gtk_ctree_node_set_row_data_full(GTK_CTREE(albero), uid_n, nd,
    (GtkDestroyNotify)g_free);
      
  while (IsntPGPError(PGPKeyIterNextUIDSig(keyIter, &sig))) {
    parse_sig(keySet, sig, uid_n, albero);
  }
}

static void parse_subkey(PGPKeySetRef keySet, PGPKeyIterRef keyIter, PGPSubKeyRef subKey, GtkCTreeNode *nodo, GtkWidget *albero) {
  PGPError err;
  PGPTime data;

  clear_buf();

  buf_par[0] = "sub key";
  
  err = PGPGetSubKeyTime(subKey, kPGPKeyPropCreation, &data);
  create_time(err, data, temp_par[3]);

  err = PGPGetSubKeyTime(subKey, kPGPKeyPropExpiration, &data);
  create_time(err, data, temp_par[4]);

  gtk_ctree_insert_node(GTK_CTREE(albero), nodo, NULL, buf_par,
    0, NULL, NULL, NULL, NULL, FALSE, FALSE);
}

static void parse_key(PGPKeySetRef keySet, PGPKeyIterRef keyIter, PGPKeyRef key, GtkCTreeNode *nodo, GtkWidget *albero) {
  PGPSize used;
  PGPError err;
  PGPInt32 num;
  PGPUserIDRef userID;
  PGPSubKeyRef subKey;
  PGPTime data;
  PGPBoolean vf;
  char outString[kPGPMaxKeyIDStringSize];
  NodeData *nd = NULL;
  
  static char pubblica[] = "public";
  static char segreta[] = "secret";

  GtkCTreeNode *key_n;
 
  clear_buf();

  err = PGPGetPrimaryUserIDNameBuffer(key, sizeof(temp_par[0]), temp_par[0], &used);
  if (IsPGPError(err)) {
    sprintf(temp_par[0], "%d", err);
  } else {
    temp_par[0][used] = '\0';
  }

  nd = g_malloc(KEYID_SIZE);
  nd->tipo = KEYID;
  err = PGPGetKeyIDFromKey(key, &nd->keyID);
  if (IsPGPError(err)) {
    sprintf(temp_par[1], "%d", err);
    g_free(nd);
    nd = NULL;
  } else {
    err = PGPGetKeyIDString(&nd->keyID, kPGPKeyIDString_Full, outString);
    if (IsPGPError(err)) {
      sprintf(temp_par[1], "%d", err);
    } else {
      buf_par[1] = outString;
    }
  }

  err = PGPGetKeyNumber(key, kPGPKeyPropBits, &num);
  if (IsPGPError(err)) {
    sprintf(temp_par[2], "%d", err);
  } else {
    sprintf(temp_par[2], "%d", num);
  }
        
  err = PGPGetKeyTime(key, kPGPKeyPropCreation, &data);
  create_time(err, data, temp_par[3]);

  err = PGPGetKeyTime(key, kPGPKeyPropExpiration, &data);
  create_time(err, data, temp_par[4]);

  err = PGPGetKeyBoolean(key, kPGPKeyPropIsSecret, &vf);
  if (IsPGPError(err)) {
    sprintf(temp_par[5], "%d", err);
  } else {
    if (vf) {
      buf_par[5] = segreta;
    } else {
      buf_par[5] = pubblica;
    }
  }

  key_n = gtk_ctree_insert_node(GTK_CTREE(albero), nodo, NULL, buf_par,
    0, NULL, NULL, NULL, NULL, FALSE, FALSE);
  gtk_ctree_node_set_row_data_full(GTK_CTREE(albero), key_n, nd,
    (GtkDestroyNotify)g_free);

  while (IsntPGPError(PGPKeyIterNextSubKey(keyIter, &subKey))) {
    parse_subkey(keySet, keyIter, subKey, key_n, albero);
  }

  while (IsntPGPError(PGPKeyIterNextUserID(keyIter, &userID))) {
    parse_userid(keySet, keyIter, userID, key_n, albero);
  }
}

void parse_key_set(PGPKeySetRef keySet, GtkCTreeNode *radice, GtkWidget *albero) {
  PGPKeyIterRef keyIter = kInvalidPGPKeyIterRef;
  PGPKeyRef key;
  PGPKeyListRef keyList;

  gtk_clist_freeze(GTK_CLIST(albero));

  PGPOrderKeySet(keySet, kPGPUserIDOrdering, &keyList);
  PGPNewKeyIter(keyList, &keyIter);

  while (IsntPGPError(PGPKeyIterNext(keyIter, &key))) {
    parse_key(keySet, keyIter, key, radice, albero);
    gtk_main_iteration_do(FALSE);
  }

  gtk_clist_thaw(GTK_CLIST(albero));

  PGPFreeKeyList(keyList);
  PGPFreeKeyIter(keyIter);
}
