#include <pgpRandomPool.h>
#include <pgpUtilities.h>

#include "varie.h"
#include "global.h"
#include "support.h"
#include "interface.h"

char temp_par[6][128];
char *buf_par[6];

void clear_buf() {
  int i;
  for (i = 0; i < 6; i++) {
    buf_par[i] = temp_par[i];
    temp_par[i][0] = '\0';
  }
}

void create_time(PGPError err, PGPTime date, char *output) {
  PGPUInt16 day, month, year; 
  if (IsPGPError(err)) {
    sprintf(output, "%d", err);
  } else {
    if (date == kPGPExpirationTime_Never) {
      sprintf(output, "never");
    } else {
      PGPGetYMDFromPGPTime(date, &year, &month, &day);
      sprintf(output, "%02d/%02d/%d", day, month, year);
    }
  }
}

void fingerprint_string(void *data, PGPSize len, char *output) {
  int i, l;
  for (i = 0; i < len; i++) {
    l = ((unsigned char *)data)[i];
    sprintf(&(output[i * 3]), "%02x ", l);
  }
}

void populate_lista(PGPKeySetRef keySet, GtkWidget *list) {
  PGPKeyIterRef keyIter = kInvalidPGPKeyIterRef;
  PGPKeyRef key;
  PGPKeyListRef keyList;
  PGPSize used;
  PGPError err;
  PGPKeyID keyID;
  char outString[kPGPMaxKeyIDStringSize];

  PGPOrderKeySet(keySet, kPGPUserIDOrdering, &keyList);
  PGPNewKeyIter(keyList, &keyIter);

  while (IsntPGPError(PGPKeyIterNext(keyIter, &key))) {

    clear_buf();

    err = PGPGetPrimaryUserIDNameBuffer(key, sizeof(temp_par[0]), temp_par[0], &used);
    if (IsPGPError(err)) {
      sprintf(temp_par[0], "%d", err);
    } else {
      temp_par[0][used] = '\0';
    }

    err = PGPGetKeyIDFromKey(key, &keyID);
    if (IsPGPError(err)) {
      sprintf(temp_par[1], "%d", err);
    } else {
      err = PGPGetKeyIDString(&keyID, kPGPKeyIDString_Full, outString);
      if (IsPGPError(err)) {
        sprintf(temp_par[1], "%d", err);
      } else {
        buf_par[1] = outString;
      }
    }
    
    gtk_clist_append(GTK_CLIST(list), buf_par);

  }

  PGPFreeKeyList(keyList);
  PGPFreeKeyIter(keyIter);
}


void view_entropy(GtkWidget *label) {
  PGPUInt32 a, b;
  char buf[128];

  a = PGPGlobalRandomPoolGetMinimumEntropy();
  b = PGPGlobalRandomPoolGetEntropy();
  sprintf(buf, "Min: %6d Cur: %6d", a, b);

  gtk_label_set_text(GTK_LABEL(label), buf);

}
